@@ -32,6 +32,7 @@ from os import environ as env
 from pathlib import Path
 from pydoc import locate
 from typing import Any
+from typing import Awaitable
 from typing import Callable
 from typing import Generic
 from typing import Iterator
@@ -69,6 +70,7 @@ from dipdup.utils import exclude_none
 from dipdup.utils import import_from
 from dipdup.utils import pascal_to_snake
 from dipdup.utils import snake_to_pascal
+from dipdup.utils.sys import is_in_tests
 
 # NOTE: ${VARIABLE:-default} | ${VARIABLE}
 ENV_VARIABLE_REGEX = r'\$\{(?P<var_name>[\w]+)(?:\:\-(?P<default_value>.*?))?\}'
@@ -171,7 +173,7 @@ class PostgresDatabaseConfig:
         }
 
     @validator('immune_tables', allow_reuse=True)
-    def _valid_immune_tables(cls, v) -> None:
+    def _valid_immune_tables(cls, v: set[str]) -> set[str]:
         for table in v:
             if table.startswith('dipdup'):
                 raise ConfigurationError("Tables with `dipdup` prefix can't be immune")
@@ -254,9 +256,17 @@ class ContractConfig(NameMixin):
         return v
 
 
-# NOTE: Don't forget `http` and `__hash__` in all datasource configs
+class DatasourceConfig(ABC, NameMixin):
+    kind: str
+    http: HTTPConfig | None
+
+    @abstractmethod
+    def __hash__(self) -> int:
+        ...
+
+
 @dataclass
-class TzktDatasourceConfig(NameMixin):
+class TzktDatasourceConfig(DatasourceConfig):
     """TzKT datasource config
 
     :param kind: always 'tzkt'
@@ -286,7 +296,7 @@ class TzktDatasourceConfig(NameMixin):
 
 
 @dataclass
-class CoinbaseDatasourceConfig(NameMixin):
+class CoinbaseDatasourceConfig(DatasourceConfig):
     """Coinbase datasource config
 
     :param kind: always 'coinbase'
@@ -326,7 +336,7 @@ class MetadataDatasourceConfig(NameMixin):
 
 
 @dataclass
-class IpfsDatasourceConfig(NameMixin):
+class IpfsDatasourceConfig(DatasourceConfig):
     """IPFS datasource config
 
     :param kind: always 'ipfs'
@@ -343,7 +353,7 @@ class IpfsDatasourceConfig(NameMixin):
 
 
 @dataclass
-class HttpDatasourceConfig(NameMixin):
+class HttpDatasourceConfig(DatasourceConfig):
     """Generic HTTP datasource config
 
     kind: always 'http'
@@ -359,7 +369,8 @@ class HttpDatasourceConfig(NameMixin):
         return hash(self.kind + self.url)
 
 
-DatasourceConfigT = (
+# NOTE: We need unions for Pydantic deserialization
+DatasourceConfigU = (
     TzktDatasourceConfig
     | CoinbaseDatasourceConfig
     | MetadataDatasourceConfig
@@ -444,11 +455,13 @@ class PatternConfig(CodegenMixin):
         cls,
         module_name: str,
         optional: bool,
+        alias: str | None,
     ) -> tuple[str, str]:
+        arg_name = pascal_to_snake(alias or f'{module_name}_origination')
         storage_cls = f'{snake_to_pascal(module_name)}Storage'
         if optional:
-            return f'{module_name}_origination', f'Optional[Origination[{storage_cls}]] = None'
-        return f'{module_name}_origination', f'Origination[{storage_cls}]'
+            return arg_name, f'Origination[{storage_cls}] | None'
+        return arg_name, f'Origination[{storage_cls}]'
 
     @classmethod
     def format_operation_argument(
@@ -463,19 +476,20 @@ class PatternConfig(CodegenMixin):
         parameter_cls = f'{snake_to_pascal(arg_name)}Parameter'
         storage_cls = f'{snake_to_pascal(module_name)}Storage'
         if optional:
-            return arg_name, f'Optional[Transaction[{parameter_cls}, {storage_cls}]] = None'
+            return arg_name, f'Transaction[{parameter_cls}, {storage_cls}] | None'
         return arg_name, f'Transaction[{parameter_cls}, {storage_cls}]'
 
     @classmethod
     def format_untyped_operation_argument(
         cls,
-        transaction_idx: int,
+        type_: str,
+        subgroup_index: int,
         optional: bool,
         alias: str | None,
     ) -> tuple[str, str]:
-        arg_name = pascal_to_snake(alias or f'transaction_{transaction_idx}')
+        arg_name = pascal_to_snake(alias or f'{type_}_{subgroup_index}')
         if optional:
-            return arg_name, 'Optional[OperationData] = None'
+            return arg_name, 'OperationData | None'
         return arg_name, 'OperationData'
 
 
@@ -506,7 +520,7 @@ ParentT = TypeVar('ParentT')
 class ParentMixin(Generic[ParentT]):
     """`parent` field for index and template configs"""
 
-    def __post_init_post_parse__(self: ParentMixin) -> None:
+    def __post_init_post_parse__(self: ParentMixin[ParentT]) -> None:
         self._parent: ParentT | None = None
 
     @property
@@ -544,30 +558,28 @@ class ParameterTypeMixin:
 
 
 @dataclass
-class TransactionIdxMixin:
-    """`transaction_idx` field to track index of operation in group
+class SubgroupIndexMixin:
+    """`subgroup_index` field to track index of operation in group
 
-    :param transaction_idx:
+    :param subgroup_index:
     """
 
     def __post_init_post_parse__(self) -> None:
-        self._transaction_idx: int | None = None
+        self._subgroup_index: int | None = None
 
     @property
-    def transaction_idx(self) -> int:
-        if self._transaction_idx is None:
+    def subgroup_index(self) -> int:
+        if self._subgroup_index is None:
             raise ConfigInitializationException
-        return self._transaction_idx
+        return self._subgroup_index
 
-    @transaction_idx.setter
-    def transaction_idx(self, value: int) -> None:
-        self._transaction_idx = value
+    @subgroup_index.setter
+    def subgroup_index(self, value: int) -> None:
+        self._subgroup_index = value
 
 
 @dataclass
-class OperationHandlerTransactionPatternConfig(
-    PatternConfig, StorageTypeMixin, ParameterTypeMixin, TransactionIdxMixin
-):
+class OperationHandlerTransactionPatternConfig(PatternConfig, StorageTypeMixin, ParameterTypeMixin, SubgroupIndexMixin):
     """Operation handler pattern config
 
     :param type: always 'transaction'
@@ -588,7 +600,7 @@ class OperationHandlerTransactionPatternConfig(
     def __post_init_post_parse__(self) -> None:
         StorageTypeMixin.__post_init_post_parse__(self)
         ParameterTypeMixin.__post_init_post_parse__(self)
-        TransactionIdxMixin.__post_init_post_parse__(self)
+        SubgroupIndexMixin.__post_init_post_parse__(self)
         if self.entrypoint and not self.destination:
             raise ConfigurationError('Transactions with entrypoint must also have destination')
 
@@ -604,9 +616,19 @@ class OperationHandlerTransactionPatternConfig(
     def iter_arguments(self) -> Iterator[tuple[str, str]]:
         if self.entrypoint:
             module_name = self.destination_contract_config.module_name
-            yield self.format_operation_argument(module_name, self.entrypoint, self.optional, self.alias)
+            yield self.format_operation_argument(
+                module_name,
+                self.entrypoint,
+                self.optional,
+                self.alias,
+            )
         else:
-            yield self.format_untyped_operation_argument(self.transaction_idx, self.optional, self.alias)
+            yield self.format_untyped_operation_argument(
+                'transaction',
+                self.subgroup_index,
+                self.optional,
+                self.alias,
+            )
 
     @cached_property
     def source_contract_config(self) -> ContractConfig:
@@ -622,7 +644,7 @@ class OperationHandlerTransactionPatternConfig(
 
 
 @dataclass
-class OperationHandlerOriginationPatternConfig(PatternConfig, StorageTypeMixin):
+class OperationHandlerOriginationPatternConfig(PatternConfig, StorageTypeMixin, SubgroupIndexMixin):
     """Origination handler pattern config
 
     :param type: always 'origination'
@@ -664,21 +686,35 @@ class OperationHandlerOriginationPatternConfig(PatternConfig, StorageTypeMixin):
         )
 
     def iter_imports(self, package: str) -> Iterator[tuple[str, str]]:
-        if self.source:
-            module_name = self.source_contract_config.module_name
+        if self.originated_contract:
+            module_name = self.originated_contract_config.module_name
         elif self.similar_to:
             module_name = self.similar_to_contract_config.module_name
-        elif self.originated_contract:
-            module_name = self.originated_contract_config.module_name
+        elif self.source:
+            yield 'dipdup.models', 'OperationData'
+            return
         else:
             raise ConfigurationError(
                 'Origination pattern must have at least one of `source`, `similar_to`, `originated_contract` fields'
             )
+
         yield 'dipdup.models', 'Origination'
         yield self.format_storage_import(package, module_name)
 
     def iter_arguments(self) -> Iterator[tuple[str, str]]:
-        yield self.format_origination_argument(self.module_name, self.optional)
+        if self.originated_contract or self.similar_to:
+            yield self.format_origination_argument(
+                self.module_name,
+                self.optional,
+                self.alias,
+            )
+        else:
+            yield self.format_untyped_operation_argument(
+                'origination',
+                self.subgroup_index,
+                self.optional,
+                self.alias,
+            )
 
     @cached_property
     def module_name(self) -> str:
@@ -723,7 +759,7 @@ class CallbackMixin(CodegenMixin):
     callback: str
 
     def __init_subclass__(cls, kind: str) -> None:
-        cls._kind = kind  # type: ignore
+        cls._kind = kind  # type: ignore[attr-defined]
 
     def __post_init_post_parse__(self) -> None:
         self._callback_fn = None
@@ -732,15 +768,15 @@ class CallbackMixin(CodegenMixin):
 
     @cached_property
     def kind(self) -> str:
-        return self._kind  # type: ignore
+        return self._kind  # type: ignore[attr-defined,no-any-return]
 
     @cached_property
-    def callback_fn(self) -> Callable:
+    def callback_fn(self) -> Callable[..., Awaitable[None]]:
         if self._callback_fn is None:
             raise ConfigInitializationException
         return self._callback_fn
 
-    def initialize_callback_fn(self, package: str):
+    def initialize_callback_fn(self, package: str) -> None:
         if self._callback_fn:
             return
         _logger.debug('Registering %s callback `%s`', self.kind, self.callback)
@@ -756,7 +792,7 @@ class HandlerConfig(CallbackMixin, ParentMixin['IndexConfig'], kind='handler'):
         ParentMixin.__post_init_post_parse__(self)
 
 
-OperationHandlerPatternConfigT = OperationHandlerOriginationPatternConfig | OperationHandlerTransactionPatternConfig
+OperationHandlerPatternConfigU = OperationHandlerOriginationPatternConfig | OperationHandlerTransactionPatternConfig
 
 
 @dataclass
@@ -767,7 +803,7 @@ class OperationHandlerConfig(HandlerConfig, kind='handler'):
     :param pattern: Filters to match operation groups
     """
 
-    pattern: tuple[OperationHandlerPatternConfigT, ...]
+    pattern: tuple[OperationHandlerPatternConfigU, ...]
 
     def iter_imports(self, package: str) -> Iterator[tuple[str, str]]:
         yield 'dipdup.context', 'HandlerContext'
@@ -827,7 +863,7 @@ class IndexTemplateConfig(NameMixin):
 
 
 @dataclass
-class IndexConfig(TemplateValuesMixin, NameMixin, SubscriptionsMixin, ParentMixin['ResolvedIndexConfigT']):
+class IndexConfig(ABC, TemplateValuesMixin, NameMixin, SubscriptionsMixin, ParentMixin['ResolvedIndexConfigU']):
     """Index config
 
     :param datasource: Alias of index datasource in `datasources` section
@@ -865,6 +901,10 @@ class IndexConfig(TemplateValuesMixin, NameMixin, SubscriptionsMixin, ParentMixi
         config_dict['datasource'].pop('http', None)
         config_dict['datasource'].pop('buffer_size', None)
 
+    @abstractmethod
+    def import_objects(self, package: str) -> None:
+        ...
+
 
 @dataclass
 class OperationIndexConfig(IndexConfig):
@@ -922,6 +962,29 @@ class OperationIndexConfig(IndexConfig):
 
         return addresses
 
+    def import_objects(self, package: str) -> None:
+        for handler_config in self.handlers:
+            handler_config.initialize_callback_fn(package)
+
+            for pattern_config in handler_config.pattern:
+                if isinstance(pattern_config, OperationHandlerTransactionPatternConfig):
+                    if not pattern_config.entrypoint:
+                        continue
+
+                    module_name = pattern_config.destination_contract_config.module_name
+                    pattern_config.initialize_parameter_cls(package, module_name, pattern_config.entrypoint)
+                    pattern_config.initialize_storage_cls(package, module_name)
+
+                elif isinstance(pattern_config, OperationHandlerOriginationPatternConfig):
+                    if not (pattern_config.originated_contract or pattern_config.similar_to):
+                        continue
+
+                    module_name = pattern_config.module_name
+                    pattern_config.initialize_storage_cls(package, module_name)
+
+                else:
+                    raise NotImplementedError
+
 
 @dataclass
 class BigMapHandlerConfig(HandlerConfig, kind='handler'):
@@ -1031,6 +1094,11 @@ class BigMapIndexConfig(IndexConfig):
         super().strip(config_dict)
         config_dict.pop('skip_history', None)
 
+    def import_objects(self, package: str) -> None:
+        for handler_config in self.handlers:
+            handler_config.initialize_callback_fn(package)
+            handler_config.initialize_big_map_type(package)
+
 
 @dataclass
 class HeadHandlerConfig(HandlerConfig, kind='handler'):
@@ -1062,6 +1130,10 @@ class HeadIndexConfig(IndexConfig):
     def last_level(self) -> int:
         return 0
 
+    def import_objects(self, package: str) -> None:
+        for handler_config in self.handlers:
+            handler_config.initialize_callback_fn(package)
+
 
 @dataclass
 class TokenTransferHandlerConfig(HandlerConfig, kind='handler'):
@@ -1091,6 +1163,10 @@ class TokenTransferIndexConfig(IndexConfig):
     first_level: int = 0
     last_level: int = 0
 
+    def import_objects(self, package: str) -> None:
+        for handler_config in self.handlers:
+            handler_config.initialize_callback_fn(package)
+
 
 @dataclass
 class EventHandlerConfig(HandlerConfig, kind='handler'):
@@ -1158,24 +1234,31 @@ class UnknownEventHandlerConfig(HandlerConfig, kind='handler'):
         yield 'event', 'UnknownEvent'
 
 
-EventHandlerConfigT = EventHandlerConfig | UnknownEventHandlerConfig
+EventHandlerConfigU = EventHandlerConfig | UnknownEventHandlerConfig
 
 
 @dataclass
 class EventIndexConfig(IndexConfig):
     kind: Literal['event']
     datasource: str | TzktDatasourceConfig
-    handlers: tuple[EventHandlerConfigT, ...] = field(default_factory=tuple)
+    handlers: tuple[EventHandlerConfigU, ...] = field(default_factory=tuple)
 
     first_level: int = 0
     last_level: int = 0
 
+    def import_objects(self, package: str) -> None:
+        for handler_config in self.handlers:
+            handler_config.initialize_callback_fn(package)
 
-ResolvedIndexConfigT = (
+            if isinstance(handler_config, EventHandlerConfig):
+                handler_config.initialize_event_type(package)
+
+
+ResolvedIndexConfigU = (
     OperationIndexConfig | BigMapIndexConfig | HeadIndexConfig | TokenTransferIndexConfig | EventIndexConfig
 )
-IndexConfigT = ResolvedIndexConfigT | IndexTemplateConfig
-HandlerPatternConfigT = OperationHandlerOriginationPatternConfig | OperationHandlerTransactionPatternConfig
+IndexConfigU = ResolvedIndexConfigU | IndexTemplateConfig
+HandlerPatternConfigU = OperationHandlerOriginationPatternConfig | OperationHandlerTransactionPatternConfig
 
 
 @dataclass
@@ -1204,7 +1287,7 @@ class HasuraConfig:
     http: HTTPConfig | None = None
 
     @validator('url', allow_reuse=True)
-    def _valid_url(cls, v):
+    def _valid_url(cls, v: str) -> str:
         parsed_url = urlparse(v)
         if not (parsed_url.scheme and parsed_url.netloc):
             raise ConfigurationError(f'`{v}` is not a valid Hasura URL')
@@ -1391,11 +1474,11 @@ class DipDupConfig:
 
     spec_version: str
     package: str
-    datasources: dict[str, DatasourceConfigT] = field(default_factory=dict)
+    datasources: dict[str, DatasourceConfigU] = field(default_factory=dict)
     database: SqliteDatabaseConfig | PostgresDatabaseConfig = SqliteDatabaseConfig(kind='sqlite')
     contracts: dict[str, ContractConfig] = field(default_factory=dict)
-    indexes: dict[str, IndexConfigT] = field(default_factory=dict)
-    templates: dict[str, ResolvedIndexConfigT] = field(default_factory=dict)
+    indexes: dict[str, IndexConfigU] = field(default_factory=dict)
+    templates: dict[str, ResolvedIndexConfigU] = field(default_factory=dict)
     jobs: dict[str, JobConfig] = field(default_factory=dict)
     hooks: dict[str, HookConfig] = field(default_factory=dict)
     hasura: HasuraConfig | None = None
@@ -1406,9 +1489,14 @@ class DipDupConfig:
     logging: LoggingValues = LoggingValues.default
 
     def __post_init_post_parse__(self) -> None:
+        if self.package != pascal_to_snake(self.package):
+            # TODO: Remove in 7.0
+            # raise ConfigurationError('Python package name must be in snake_case.')
+            _logger.warning('Python package name must be in snake_case.')
+
         self.paths: list[Path] = []
         self.environment: dict[str, str] = {}
-        self._callback_patterns: dict[str, list[Sequence[HandlerPatternConfigT]]] = defaultdict(list)
+        self._callback_patterns: dict[str, list[Sequence[HandlerPatternConfigU]]] = defaultdict(list)
         self._contract_addresses = {contract.address for contract in self.contracts.values()}
 
     @cached_property
@@ -1421,9 +1509,15 @@ class DipDupConfig:
     @cached_property
     def package_path(self) -> Path:
         """Absolute path to the indexer package, existing or default"""
+        # NOTE: Integration tests run in isolated environment
+        if is_in_tests():
+            return Path.cwd() / self.package
+
         with suppress(ImportError):
             package = importlib.import_module(self.package)
-            return Path(cast(str, package.__file__)).parent
+            if package.__file__ is None:
+                raise RuntimeError(f'`{package.__name__}` package has no `__file__` attribute')
+            return Path(package.__file__).parent
 
         # NOTE: Detect src/<package> layout
         if Path('src').is_dir():
@@ -1446,6 +1540,7 @@ class DipDupConfig:
         paths: list[Path],
         environment: bool = True,
     ) -> DipDupConfig:
+        # NOTE: __future__.annotations
         JobConfig.__pydantic_model__.update_forward_refs()  # type: ignore[attr-defined]
 
         yaml = YAML(typ='base')
@@ -1463,10 +1558,13 @@ class DipDupConfig:
 
         try:
             config = cls(**json_config)
-            config.environment = config_environment
-            config.paths = paths
+        except ConfigurationError:
+            raise
         except Exception as e:
             raise ConfigurationError(str(e)) from e
+
+        config.environment = config_environment
+        config.paths = paths
         return config
 
     def dump(self) -> str:
@@ -1486,19 +1584,19 @@ class DipDupConfig:
         except KeyError as e:
             raise ConfigurationError(f'Contract `{name}` not found in `contracts` config section') from e
 
-    def get_datasource(self, name: str) -> DatasourceConfigT:
+    def get_datasource(self, name: str) -> DatasourceConfigU:
         try:
             return self.datasources[name]
         except KeyError as e:
             raise ConfigurationError(f'Datasource `{name}` not found in `datasources` config section') from e
 
-    def get_index(self, name: str) -> IndexConfigT:
+    def get_index(self, name: str) -> IndexConfigU:
         try:
             return self.indexes[name]
         except KeyError as e:
             raise ConfigurationError(f'Index `{name}` not found in `indexes` config section') from e
 
-    def get_template(self, name: str) -> ResolvedIndexConfigT:
+    def get_template(self, name: str) -> ResolvedIndexConfigU:
         try:
             return self.templates[name]
         except KeyError as e:
@@ -1527,15 +1625,6 @@ class DipDupConfig:
         if isinstance(self.package, str):
             logging.getLogger(self.package).setLevel(level)
 
-    def _import_index(self, index_config: IndexConfigT) -> None:
-        _logger.debug('Loading callbacks and typeclasses of index `%s`', index_config.name)
-
-        if isinstance(index_config, IndexTemplateConfig):
-            raise ConfigInitializationException
-
-        self._import_index_types(index_config)
-        self._import_index_callbacks(index_config)
-
     def initialize(self, skip_imports: bool = False) -> None:
         self._set_names()
         self._resolve_templates()
@@ -1546,7 +1635,9 @@ class DipDupConfig:
             return
 
         for index_config in self.indexes.values():
-            self._import_index(index_config)
+            if isinstance(index_config, IndexTemplateConfig):
+                raise ConfigInitializationException
+            index_config.import_objects(self.package)
 
     def add_index(self, name: str, template: str, values: dict[str, str]) -> None:
         if name in self.indexes:
@@ -1557,11 +1648,11 @@ class DipDupConfig:
         )
         template_config.name = name
         self._resolve_template(template_config)
-        index_config = cast(ResolvedIndexConfigT, self.indexes[name])
+        index_config = cast(ResolvedIndexConfigU, self.indexes[name])
         self._resolve_index_links(index_config)
         self._resolve_index_subscriptions(index_config)
         index_config.name = name
-        self._import_index(index_config)
+        index_config.import_objects(self.package)
 
     @classmethod
     def _load_raw_config(cls, path: Path) -> str:
@@ -1664,7 +1755,7 @@ class DipDupConfig:
                     raise ConfigurationError('`HookConfig.atomic` and `JobConfig.daemon` flags are mutually exclusive')
                 job_config.hook = hook_config
 
-    def _resolve_index_subscriptions(self, index_config: IndexConfigT) -> None:
+    def _resolve_index_subscriptions(self, index_config: IndexConfigU) -> None:
         if isinstance(index_config, IndexTemplateConfig):
             return
         if index_config.subscriptions:
@@ -1721,7 +1812,12 @@ class DipDupConfig:
         else:
             raise NotImplementedError(f'Index kind `{index_config.kind}` is not supported')
 
-    def _resolve_index_links(self, index_config: ResolvedIndexConfigT) -> None:
+        if not index_config.subscriptions:
+            raise ConfigurationError(
+                f'`{index_config.name}` index has no subscriptions; ensure that config is correct.'
+            )
+
+    def _resolve_index_links(self, index_config: ResolvedIndexConfigU) -> None:
         """Resolve contract and datasource configs by aliases"""
         # NOTE: Each index must have a corresponding (currently) TzKT datasource
         if isinstance(index_config.datasource, str):
@@ -1737,14 +1833,14 @@ class DipDupConfig:
                 handler_config.parent = index_config
                 self._callback_patterns[handler_config.callback].append(handler_config.pattern)
                 for idx, pattern_config in enumerate(handler_config.pattern):
-                    # NOTE: Untyped operations are named as `transaction_N` based on their index
+                    # NOTE: Untyped operations are named as `transaction_N` or `origination_N` based on their index
+                    pattern_config._subgroup_index = idx
+
                     if isinstance(pattern_config, OperationHandlerTransactionPatternConfig):
                         if isinstance(pattern_config.destination, str):
                             pattern_config.destination = self.get_contract(pattern_config.destination)
                         if isinstance(pattern_config.source, str):
                             pattern_config.source = self.get_contract(pattern_config.source)
-                        if not pattern_config.entrypoint:
-                            pattern_config._transaction_idx = idx
 
                     elif isinstance(pattern_config, OperationHandlerOriginationPatternConfig):
                         if isinstance(pattern_config.source, str):
@@ -1800,33 +1896,3 @@ class DipDupConfig:
         for named_configs in named_config_sections:
             for name, config in named_configs.items():
                 config.name = name
-
-    def _import_index_callbacks(self, index_config: ResolvedIndexConfigT) -> None:
-        for handler_config in index_config.handlers:
-            handler_config.initialize_callback_fn(self.package)
-
-    def _import_index_types(self, index_config: ResolvedIndexConfigT) -> None:
-        if isinstance(index_config, OperationIndexConfig):
-            for handler_config in index_config.handlers:
-                for pattern_config in handler_config.pattern:
-                    if isinstance(pattern_config, OperationHandlerTransactionPatternConfig):
-                        if pattern_config.entrypoint:
-                            module_name = pattern_config.destination_contract_config.module_name
-                            pattern_config.initialize_parameter_cls(
-                                self.package, module_name, pattern_config.entrypoint
-                            )
-                            pattern_config.initialize_storage_cls(self.package, module_name)
-                    elif isinstance(pattern_config, OperationHandlerOriginationPatternConfig):
-                        module_name = pattern_config.module_name
-                        pattern_config.initialize_storage_cls(self.package, module_name)
-                    else:
-                        raise NotImplementedError
-
-        elif isinstance(index_config, BigMapIndexConfig):
-            for big_map_handler_config in index_config.handlers:
-                big_map_handler_config.initialize_big_map_type(self.package)
-
-        elif isinstance(index_config, EventIndexConfig):
-            for event_handler_config in index_config.handlers:
-                if isinstance(event_handler_config, EventHandlerConfig):
-                    event_handler_config.initialize_event_type(self.package)
