@@ -13,6 +13,7 @@ from typing import Dict
 from typing import Generic
 from typing import Iterable
 from typing import Iterator
+from typing import NoReturn
 from typing import Optional
 from typing import Sequence
 from typing import Set
@@ -28,18 +29,16 @@ from dipdup.config import BigMapHandlerConfig
 from dipdup.config import BigMapIndexConfig
 from dipdup.config import ContractConfig
 from dipdup.config import EventHandlerConfig
-from dipdup.config import EventHandlerConfigT
+from dipdup.config import EventHandlerConfigU
 from dipdup.config import EventIndexConfig
 from dipdup.config import HeadHandlerConfig
 from dipdup.config import HeadIndexConfig
 from dipdup.config import OperationHandlerConfig
 from dipdup.config import OperationHandlerOriginationPatternConfig
-from dipdup.config import OperationHandlerPatternConfigT
+from dipdup.config import OperationHandlerPatternConfigU
 from dipdup.config import OperationHandlerTransactionPatternConfig
 from dipdup.config import OperationIndexConfig
-from dipdup.config import OperationType
-from dipdup.config import ResolvedIndexConfigT
-from dipdup.config import SkipHistory
+from dipdup.config import ResolvedIndexConfigU
 from dipdup.config import TokenTransferHandlerConfig
 from dipdup.config import TokenTransferIndexConfig
 from dipdup.config import UnknownEventHandlerConfig
@@ -52,7 +51,10 @@ from dipdup.datasources.tzkt.fetcher import EventFetcher
 from dipdup.datasources.tzkt.fetcher import OperationFetcher
 from dipdup.datasources.tzkt.fetcher import TokenTransferFetcher
 from dipdup.datasources.tzkt.models import deserialize_storage
+from dipdup.enums import IndexStatus
 from dipdup.enums import MessageType
+from dipdup.enums import OperationType
+from dipdup.enums import SkipHistory
 from dipdup.exceptions import ConfigInitializationException
 from dipdup.exceptions import ConfigurationError
 from dipdup.exceptions import InvalidDataError
@@ -62,7 +64,6 @@ from dipdup.models import BigMapDiff
 from dipdup.models import Event
 from dipdup.models import EventData
 from dipdup.models import HeadBlockData
-from dipdup.models import IndexStatus
 from dipdup.models import OperationData
 from dipdup.models import Origination
 from dipdup.models import TokenTransferData
@@ -74,7 +75,7 @@ from dipdup.utils.codegen import parse_object
 
 _logger = logging.getLogger(__name__)
 
-ConfigT = TypeVar('ConfigT', bound=ResolvedIndexConfigT)
+ConfigT = TypeVar('ConfigT', bound=ResolvedIndexConfigU)
 
 
 @dataclass(frozen=True)
@@ -86,9 +87,6 @@ class OperationSubgroup:
     operations: Tuple[OperationData, ...]
     entrypoints: Set[Optional[str]]
 
-    def __hash__(self) -> int:
-        return hash(f'{self.hash}:{self.counter}')
-
 
 OperationHandlerArgumentT = Optional[Union[Transaction, Origination, OperationData]]
 MatchedOperationsT = Tuple[OperationSubgroup, OperationHandlerConfig, Deque[OperationHandlerArgumentT]]
@@ -154,7 +152,7 @@ class Index(Generic[ConfigT]):
     """
 
     message_type: MessageType
-    _queue: Deque
+    _queue: Deque[Any]
 
     def __init__(self, ctx: DipDupContext, config: ConfigT, datasource: TzktDatasource) -> None:
         self._ctx = ctx
@@ -190,9 +188,9 @@ class Index(Generic[ConfigT]):
         """Get level index needs to be synchronized to depending on its subscription status"""
         sync_levels = {self.datasource.get_sync_level(s) for s in self._config.subscriptions}
         if not sync_levels:
-            raise RuntimeError('Index has no subscriptions')
+            raise RuntimeError('Initialize config before starting `IndexDispatcher`')
         if None in sync_levels:
-            raise RuntimeError('Call `set_sync_level` before starting IndexDispatcher')
+            raise RuntimeError('Call `set_sync_level` before starting `IndexDispatcher`')
         # NOTE: Multiple sync levels means index with new subscriptions was added in runtime.
         # NOTE: Choose the highest level; outdated realtime messages will be dropped from the queue anyway.
         return max(cast(Set[int], sync_levels))
@@ -264,7 +262,7 @@ class Index(Generic[ConfigT]):
         ...
 
     @abstractmethod
-    async def _create_fetcher(self, first_level: int, last_level: int) -> DataFetcher:
+    async def _create_fetcher(self, first_level: int, last_level: int) -> DataFetcher[Any]:
         ...
 
     async def _enter_sync_state(self, head_level: int) -> Optional[int]:
@@ -423,7 +421,7 @@ class OperationIndex(Index[OperationIndexConfig]):
                 await self._call_matched_handler(handler_config, operation_subgroup, args)
             await self.state.update_status(level=batch_level)
 
-    async def _match_operation(self, pattern_config: OperationHandlerPatternConfigT, operation: OperationData) -> bool:
+    async def _match_operation(self, pattern_config: OperationHandlerPatternConfigU, operation: OperationData) -> bool:
         """Match single operation with pattern"""
         # NOTE: Reversed conditions are intentional
         if isinstance(pattern_config, OperationHandlerTransactionPatternConfig):
@@ -465,14 +463,14 @@ class OperationIndex(Index[OperationIndexConfig]):
         operations = operation_subgroup.operations
 
         for handler_config in self._config.handlers:
-            operation_idx = 0
+            subgroup_index = 0
             pattern_idx = 0
             matched_operations: Deque[Optional[OperationData]] = deque()
 
             # TODO: Ensure complex cases work, e.g. when optional argument is followed by required one
             # TODO: Add None to matched_operations where applicable (pattern is optional and operation not found)
-            while operation_idx < len(operations):
-                operation, pattern_config = operations[operation_idx], handler_config.pattern[pattern_idx]
+            while subgroup_index < len(operations):
+                operation, pattern_config = operations[subgroup_index], handler_config.pattern[pattern_idx]
                 operation_matched = await self._match_operation(pattern_config, operation)
 
                 if operation.type == 'origination' and isinstance(
@@ -487,12 +485,12 @@ class OperationIndex(Index[OperationIndexConfig]):
                 if operation_matched:
                     matched_operations.append(operation)
                     pattern_idx += 1
-                    operation_idx += 1
+                    subgroup_index += 1
                 elif pattern_config.optional:
                     matched_operations.append(None)
                     pattern_idx += 1
                 else:
-                    operation_idx += 1
+                    subgroup_index += 1
 
                 if pattern_idx == len(handler_config.pattern):
                     self._logger.info('%s: `%s` handler matched!', operation_subgroup.hash, handler_config.callback)
@@ -533,22 +531,26 @@ class OperationIndex(Index[OperationIndexConfig]):
                 storage_type = pattern_config.storage_type_cls
                 storage = deserialize_storage(operation_data, storage_type)
 
-                transaction_context: Transaction[Any, Any] = Transaction(
+                typed_transaction: Transaction[Any, Any] = Transaction(
                     data=operation_data,
                     parameter=parameter,
                     storage=storage,
                 )
-                args.append(transaction_context)
+                args.append(typed_transaction)
 
             elif isinstance(pattern_config, OperationHandlerOriginationPatternConfig):
+                if not (pattern_config.originated_contract or pattern_config.similar_to):
+                    args.append(operation_data)
+                    continue
+
                 storage_type = pattern_config.storage_type_cls
                 storage = deserialize_storage(operation_data, storage_type)
 
-                origination_context = Origination(
+                typed_origination = Origination(
                     data=operation_data,
                     storage=storage,
                 )
-                args.append(origination_context)
+                args.append(typed_origination)
 
             else:
                 raise NotImplementedError
@@ -611,7 +613,7 @@ class OperationIndex(Index[OperationIndexConfig]):
         return self._contract_hashes[address]
 
 
-class BigMapIndex(Index):
+class BigMapIndex(Index[BigMapIndexConfig]):
     message_type = MessageType.big_map
 
     def __init__(self, ctx: DipDupContext, config: BigMapIndexConfig, datasource: TzktDatasource) -> None:
@@ -640,7 +642,7 @@ class BigMapIndex(Index):
                     stack.enter_context(Metrics.measure_level_realtime_duration())
                 await self._process_level_big_maps(big_maps, message_level)
 
-    async def _create_fetcher(self, first_level: int, last_level: int) -> DataFetcher:
+    async def _create_fetcher(self, first_level: int, last_level: int) -> BigMapFetcher:
         big_map_addresses = self._get_big_map_addresses()
         big_map_paths = self._get_big_map_paths()
 
@@ -764,7 +766,7 @@ class BigMapIndex(Index):
         self,
         handler_config: BigMapHandlerConfig,
         matched_big_map: BigMapData,
-    ) -> BigMapDiff:
+    ) -> BigMapDiff[Any, Any]:
         """Prepare handler arguments, parse key and value. Schedule callback in executor."""
         self._logger.info('%s: `%s` handler matched!', matched_big_map.operation_id, handler_config.callback)
 
@@ -800,7 +802,9 @@ class BigMapIndex(Index):
 
         return matched_handlers
 
-    async def _call_matched_handler(self, handler_config: BigMapHandlerConfig, big_map_diff: BigMapDiff) -> None:
+    async def _call_matched_handler(
+        self, handler_config: BigMapHandlerConfig, big_map_diff: BigMapDiff[Any, Any]
+    ) -> None:
         if not handler_config.parent:
             raise ConfigInitializationException
 
@@ -840,14 +844,14 @@ class BigMapIndex(Index):
         return pairs
 
 
-class HeadIndex(Index):
+class HeadIndex(Index[HeadIndexConfig]):
     message_type: MessageType = MessageType.head
 
     def __init__(self, ctx: DipDupContext, config: HeadIndexConfig, datasource: TzktDatasource) -> None:
         super().__init__(ctx, config, datasource)
         self._queue: Deque[HeadBlockData] = deque()
 
-    async def _create_fetcher(self, first_level: int, last_level: int) -> DataFetcher:
+    async def _create_fetcher(self, first_level: int, last_level: int) -> NoReturn:
         raise NotImplementedError('HeadIndex has no fetcher')
 
     async def _synchronize(self, sync_level: int) -> None:
@@ -891,7 +895,7 @@ class HeadIndex(Index):
         self._queue.append(head)
 
 
-class TokenTransferIndex(Index):
+class TokenTransferIndex(Index[TokenTransferIndexConfig]):
     message_type = MessageType.token_transfer
 
     def __init__(self, ctx: DipDupContext, config: TokenTransferIndexConfig, datasource: TzktDatasource) -> None:
@@ -904,7 +908,7 @@ class TokenTransferIndex(Index):
         if Metrics.enabled:
             Metrics.set_levels_to_realtime(self._config.name, len(self._queue))
 
-    async def _create_fetcher(self, first_level: int, last_level: int) -> DataFetcher:
+    async def _create_fetcher(self, first_level: int, last_level: int) -> TokenTransferFetcher:
         token_addresses: set[str] = set()
         token_ids: set[int] = set()
         from_addresses: set[str] = set()
@@ -1043,7 +1047,7 @@ class TokenTransferIndex(Index):
                 await self._process_level_token_transfers(token_transfers, message_level)
 
 
-class EventIndex(Index):
+class EventIndex(Index[EventIndexConfig]):
     message_type = MessageType.event
 
     def __init__(self, ctx: DipDupContext, config: EventIndexConfig, datasource: TzktDatasource) -> None:
@@ -1072,7 +1076,7 @@ class EventIndex(Index):
                     stack.enter_context(Metrics.measure_level_realtime_duration())
                 await self._process_level_events(events, message_level)
 
-    async def _create_fetcher(self, first_level: int, last_level: int) -> DataFetcher:
+    async def _create_fetcher(self, first_level: int, last_level: int) -> EventFetcher:
         event_addresses = self._get_event_addresses()
         event_tags = self._get_event_tags()
         return EventFetcher(
@@ -1131,7 +1135,7 @@ class EventIndex(Index):
                 await self._call_matched_handler(handler_config, event)
             await self.state.update_status(level=batch_level)
 
-    async def _match_event(self, handler_config: EventHandlerConfigT, event: EventData) -> bool:
+    async def _match_event(self, handler_config: EventHandlerConfigU, event: EventData) -> bool:
         """Match single contract event with pattern"""
         if isinstance(handler_config, EventHandlerConfig) and handler_config.tag != event.tag:
             return False
@@ -1141,9 +1145,9 @@ class EventIndex(Index):
 
     async def _prepare_handler_args(
         self,
-        handler_config: EventHandlerConfigT,
+        handler_config: EventHandlerConfigU,
         matched_event: EventData,
-    ) -> Event | UnknownEvent | None:
+    ) -> Event[Any] | UnknownEvent | None:
         """Prepare handler arguments, parse key and value. Schedule callback in executor."""
         self._logger.info('%s: `%s` handler matched!', matched_event.level, handler_config.callback)
 
@@ -1193,7 +1197,9 @@ class EventIndex(Index):
 
         return matched_handlers
 
-    async def _call_matched_handler(self, handler_config: EventHandlerConfigT, event: Event | UnknownEvent):
+    async def _call_matched_handler(
+        self, handler_config: EventHandlerConfigU, event: Event[Any] | UnknownEvent
+    ) -> None:
         if isinstance(handler_config, EventHandlerConfig) != isinstance(event, Event):
             raise RuntimeError(f'Invalid handler config and event types: {handler_config}, {event}')
 
