@@ -6,6 +6,7 @@ import logging
 from contextlib import asynccontextmanager
 from contextlib import suppress
 from pathlib import Path
+from types import ModuleType
 from typing import Any
 from typing import AsyncIterator
 from typing import Dict
@@ -17,17 +18,17 @@ from typing import Tuple
 from typing import Type
 from typing import Union
 
-import sqlparse  # type: ignore
-from asyncpg import CannotConnectNowError  # type: ignore
-from tortoise import ForeignKeyFieldInstance
-from tortoise import Model as TortoiseModel
-from tortoise import ModuleType
+import sqlparse  # type: ignore[import]
+from asyncpg import CannotConnectNowError  # type: ignore[import]
 from tortoise import Tortoise
-from tortoise import connections
 from tortoise.backends.asyncpg.client import AsyncpgDBClient
 from tortoise.backends.base.client import BaseDBAsyncClient
+from tortoise.backends.base.executor import EXECUTOR_CACHE
 from tortoise.backends.sqlite.client import SqliteClient
+from tortoise.connection import connections
 from tortoise.fields import DecimalField
+from tortoise.fields.relational import ForeignKeyFieldInstance
+from tortoise.models import Model as TortoiseModel
 from tortoise.utils import get_schema_sql
 
 from dipdup.exceptions import DatabaseEngineError
@@ -51,7 +52,7 @@ def set_connection(conn: BaseDBAsyncClient) -> None:
 
 
 @asynccontextmanager
-async def tortoise_wrapper(url: str, models: Optional[str] = None, timeout: int = 60) -> AsyncIterator:
+async def tortoise_wrapper(url: str, models: Optional[str] = None, timeout: int = 60) -> AsyncIterator[None]:
     """Initialize Tortoise with internal and project models, close connections when done"""
     model_modules: Dict[str, Iterable[Union[str, ModuleType]]] = {
         'int_models': ['dipdup.models'],
@@ -258,6 +259,9 @@ def prepare_models(package: Optional[str]) -> None:
     # NOTE: Circular imports
     import dipdup.models
 
+    # NOTE: Required for pytest-xdist. Models with the same name in different packages cause conflicts otherwise.
+    EXECUTOR_CACHE.clear()
+
     db_tables: Set[str] = set()
     decimal_context = decimal.getcontext()
     prec = decimal_context.prec
